package fi.jyu.mit.fxgui;

import java.util.ArrayList;

import javafx.beans.property.BooleanProperty;
import javafx.beans.property.SimpleBooleanProperty;
import javafx.beans.value.ChangeListener;
import javafx.scene.control.RadioButton;
import javafx.scene.control.ToggleGroup;

/**
 * Allows multiple radiobuttons to be selected and managed with only 1 component
 * @author terop
 * @version 13.1.2017
 * @param <T> Type to be stored in to this class
 */
public class RadioButtonChooser<T> extends BaseBoxChooser<T, RadioButton> {

    private ToggleGroup group = new ToggleGroup();
    /**
     * Should multiple selections be allowed
     */
    protected BooleanProperty multipleSelectionsAllowed = new SimpleBooleanProperty(false);
    
	@Override
	protected void addChangeListener(RadioButton box, ChangeListener<Boolean> listener) {
		box.selectedProperty().addListener(listener);
	}

	@Override
	protected RadioButton addCorrectComponent(String name, T object) {
		RadioButton button = new RadioButton(name);
		if(!multipleSelectionsAllowed.get())
		    button.setToggleGroup(group);
		button.setMnemonicParsing(true);
		return button;
	}
	
	
	@Override
	protected void removeListener(RadioButton node, ArrayList<ChangeListener<Boolean>> list) {
		for (ChangeListener<Boolean> changeListener : list) {
			node.selectedProperty().removeListener(changeListener);
		}
		
	}
	
	/**
	 * Sets if selecting multiple toggles is allowed
	 * @param allowMultiples if multiples are allowed
	 */
	public void setMultipleSelectionsAllowed(boolean allowMultiples){
	    this.multipleSelectionsAllowed.set(allowMultiples);
	    UpdateComponents(getItems());
	}

	/**
	 * Gets the multiple selection allowed propertys value
	 * @return if multiple selections are allowed
	 */
	public boolean getMultipleSelectionsAllowed(){
        return this.multipleSelectionsAllowed.get();
    }
	

	@Override
	protected boolean isComponentSelected(RadioButton component) {
		return component.selectedProperty().get();
	}


    @Override
    protected boolean setComponentSelected(RadioButton component, boolean value) {
        boolean oldSelect = isComponentSelected(component);
        component.selectedProperty().set(value);
        return oldSelect;
    }
    
	
}
