package application;

import application.controllers.FirstViewController;
import javafx.application.Application;
import javafx.fxml.FXMLLoader;
import javafx.scene.Scene;
import javafx.scene.layout.BorderPane;
import javafx.stage.Stage;

/**
 * Main class for Test program. Meant for testing purposes of JavaFX for the
 * Programming 2 course in University of Jyvskyl.
 * 
 * @author Petri Monola
 * @version 20.4.2017
 */
public class ExampleMain extends Application {

    /**
     * Start method for launching the application.
     * @param primaryStage Primary stage of the application
     */
    @Override
    public void start(Stage primaryStage) {
        try {
            FXMLLoader ldr = new FXMLLoader(getClass().getResource("views/FirstView.fxml")); // Loader fetches the resource(starting view) for application
            BorderPane root = (BorderPane) ldr.load(); // Load resource into a BorderPane (same root element as defined in the fxml document)

            Scene scene = new Scene(root, 400, 400); // Setup scene, and set size for window (400 x 400 pixels)
            scene.getStylesheets().add(getClass().getResource("styles.css").toExternalForm()); // Load style sheets

            primaryStage.setScene(scene); // Add scene for the stage (application)
            primaryStage.setResizable(true); // Set whether we want to resize the window
            primaryStage.setTitle("JavaFX test"); // Set title for application
            primaryStage.show(); // Draw stage (application) on the screen

            FirstViewController ctrl = (FirstViewController) ldr.getController(); // Extract controller from loader
            ctrl.setWidthListener(); // Calling a method defined in the controller
        } catch (Exception e) {
            e.printStackTrace(); // If something went wrong, print error
        }
    }

    /**
     * Main method. Launches the application.
     * @param args Launch arguments
     */
    public static void main(String[] args) {
        launch(args);
    }
}
