package application.controllers;

import java.net.URL;
import java.util.ResourceBundle;

import application.objects.Olio;
import fi.jyu.mit.fxgui.ComboBoxChooser;
import fi.jyu.mit.fxgui.Dialogs;
import fi.jyu.mit.fxgui.ModalController;
import fi.jyu.mit.fxgui.ModalControllerInterface;
import javafx.fxml.FXML;
import javafx.fxml.Initializable;
import javafx.scene.control.Label;
import javafx.scene.control.RadioButton;
import javafx.scene.control.ToggleGroup;
import javafx.scene.input.MouseEvent;

/**
 * Controller for SecondView. Meant for testing purposes of JavaFX for the
 * Programming 2 course in University of Jyvskyl.
 * 
 * @author Petri Monola
 * @version 20.4.2017
 *
 */
public class SecondViewController implements Initializable, ModalControllerInterface<Olio> {

    
    
/* METHODS AND FIELDS THAT ARE RELATED TO FXML                              */
/* ======================================================================== */

    
    /* Fields defined in the fxml file. */
    @FXML private Label labelChosen;
    @FXML private ToggleGroup group;

    /* Defining type Olio for the ComboBoxChooser so that we can store Olio objects in there. */
    @FXML private ComboBoxChooser<Olio> cbChooser;

    /**
     * Handler for pressing the "OLIO" button. Closes the stage.
     */
    @FXML void handleOlio() {
        RadioButton rb = (RadioButton) group.getSelectedToggle();
        int index = Integer.parseInt(rb.getId()); /* Gets the id of RadioButton. Works only
                                                   * if RadioButton's id is an integer */

        /* Set result to be different based on the RadioButton choice */
        switch (index) {
        case 0:
            result = new Olio(20, "rb4sd", "sgexer");
            break;
        case 1:
            result = new Olio(50, "hge3", "dgh4");
            break;
        default:
            result = new Olio(1, "jhfse", "rh4hd");
            break;
        }

        ModalController.closeStage(labelChosen); /* Closes this view. Needs an element 
                                                  * from the view as a parameter */ 
    }

    /**
     * Handles a click event. Reacts only to double clicks.
     * @param event MouseEvent
     */
    @FXML private void handleClick(MouseEvent event) {
        if (event.getClickCount() == 2) {
            System.err.println("Clicked twice!");
            Dialogs.showMessageDialog("Doesn't really do much, but at least you clicked twice!");
        }
    }

    
    
/* METHODS THAT ARE RELATED TO THE INTERFACES                               */
/* ======================================================================== */

    
    /**
     * Initialize method. Called upon launching the view. Can be used to setup
     * listeners or formatting fields.
     */
    @Override
    public void initialize(URL location, ResourceBundle resources) {
        group.selectedToggleProperty().addListener((e) -> show());
        labelChosen.setText("");
    }

    /**
     * The view returns what is defined here. Returns result based on the choice
     * from the ToggleGroup. Called upon closing the view.
     */
    @Override
    public Olio getResult() {
        return result;
    }

    /**
     * Handler for after the drawing of view on screen. Can be used for example
     * to change focus on fields.
     */
    @Override
    public void handleShown() {
        group.selectToggle(group.getToggles().get(0));
    }

    /**
     * Sets a default value given as a parameter for the view.
     */
    @Override
    public void setDefault(Olio arg0) {
        if (arg0 != null) { // Check whether the given parameter was not null
            cbChooser.add("This is an Olio object - " + arg0.toString(), arg0); // Adding an Olio to ComboBoxChooser
            cbChooser.getSelectionModel().selectFirst(); // Select first object from the chooser
        }
    }

    
    
/* METHODS AND FIELDS FOR THIS CLASS (ALL OTHER)                            */
/* ======================================================================== */

    
    /* Result returned upon pressing OLIO button */
    private Olio result;

    /**
     * Method called in the group's toggle listener. Changes the text of the
     * label according to the choice.
     */
    private void show() {
        RadioButton rb = (RadioButton) group.getSelectedToggle();
        if (rb == null)
            return;
        labelChosen.setText("Chosen: " + rb.getText().replaceFirst("_", ""));
    }
}
