package application.objects;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Random;

/**
 * Class for test object. Has some non-descriptive fields. Meant for testing
 * purposes of JavaFX for the Programming 2 course in University of Jyvskyl.
 * 
 * @author Petri
 * @version 20.4.2017
 */
public class Olio {
    private static final Random RANDOM = new Random();

    /* Fields for Olio */
    public int kentta1;
    public String kentta2;
    public Calendar kentta3;
    public String kentta4;

    /**
     * Constructor for Olio
     * 
     * @param i Some non-descriptive integer
     * @param s1 Some non-descriptive String
     * @param d Some non-descriptive double
     * @param s2 Some non-descriptive String
     */
    public Olio(int i, String s1, String s2) {
        this.kentta1 = i;
        this.kentta2 = s1;
        this.kentta3 = Calendar.getInstance();
        setRandomDate();
        this.kentta4 = s2;
    }

    /**
     * Return randomly a negative or positive sign.
     * @return -1 or 1 randomly.
     */
    private static int getRandomSign() {
        double seed = RANDOM.nextDouble();
        if (seed < 0.5)
            return -1;
        return 1;
    }

    /**
     * Sets a random date for 3rd field.
     */
    public void setRandomDate() {
        this.kentta3.set(
                kentta3.get(Calendar.YEAR) + getRandomSign() * RANDOM.nextInt(10), 
                RANDOM.nextInt(12),
                RANDOM.nextInt(31));
    }

    /**
     * Sets the value of a field according to the "index" of the fields.
     * @param i Index
     * @param s Value to set
     */
    public void setFieldValue(int i, String s) {
        switch (i) {
        case 0:
            this.kentta1 = Integer.parseInt(s);
            break;
        case 1:
            this.kentta2 = s;
            break;
        case 2:
            SimpleDateFormat sdf = new SimpleDateFormat("d.M.YYYY");
            try {
                sdf.parse(s);
            } catch (ParseException e) {
                System.err.println("Parse Exception!");
            }
            this.kentta3 = sdf.getCalendar();
            break;
        case 3:
            this.kentta4 = s;
            break;
        }
    }
    
    /**
     * Returns the field according to the "index" of the field. Used for sorting
     * purposes, especially date (3rd) field.
     * @param i "index" of the field.
     * @return Olio's field as a String.
     */
    public String getComparableString(int i) {
        switch (i) {
        case 0:
            return Integer.toString(this.kentta1);
        case 1:
            return this.kentta2;
        case 2:
            return new SimpleDateFormat("YYYY-MM-dd").format(kentta3.getTime());
        case 3:
            return this.kentta4;
        }

        return "unknown field";
    }

    /**
     * Return the field according to the "index" of the field. Used for
     * displaying the field as a string (in a grid).
     * @param i "index" of the field.
     * @return Olio's field as a String.
     */
    public String getVisibleString(int i) {
        switch (i) {
        case 0:
            return Integer.toString(this.kentta1);
        case 1:
            return this.kentta2;
        case 2:
            return new SimpleDateFormat("d.M.YYYY").format(kentta3.getTime());
        case 3:
            return this.kentta4;
        }

        return "unknown field";
    }
}
